<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Config;

use Dedi\SyliusMigrationPlugin\Dto\MigrationOption;
use Dedi\SyliusMigrationPlugin\Registry\MigrationActionRegistry;
use Dedi\SyliusMigrationPlugin\Service\MigrationAction\MigrationActionInterface;
use Dedi\SyliusMigrationPlugin\Service\MigrationAction\PostMigrationActionInterface;
use Dedi\SyliusMigrationPlugin\Service\MigrationAction\PreMigrationActionInterface;

class MigrationActionConfig
{
    /** @var array|MigrationActionInterface[] */
    private array $preMigrationsActions = [];
    /** @var array|MigrationActionInterface[] */
    private array $postMigrationsActions = [];
    private MigrationActionRegistry $migrationActionRegistry;

    public function __construct(MigrationActionRegistry $migrationActionRegistry)
    {
        $this->migrationActionRegistry = $migrationActionRegistry;
    }

    public function addMigrationAction(string $type, array $options): void
    {
        $action = $this->migrationActionRegistry->getMigrationAction($type);
        $action->setOptions($options);

        if ($action instanceof PreMigrationActionInterface) {
            $this->preMigrationsActions[] = $action;

            return;
        }

        if ($action instanceof PostMigrationActionInterface) {
            $this->postMigrationsActions[] = $action;

            return;
        }

        throw new \LogicException(sprintf('Unrecognized action %s : action must implement one of PreMigrationActionInterface, PostMigrationActionInterface', $type));
    }

    public function getPreMigrationsActions(): array
    {
        return $this->preMigrationsActions;
    }

    public function getPostMigrationsActions(): array
    {
        return $this->postMigrationsActions;
    }

    public function executePreMigrationActions(MigrationOption $migrationOption)
    {
        foreach ($this->preMigrationsActions as $preMigrationsAction) {
            $preMigrationsAction->execute($migrationOption);
        }
    }

    public function executePostMigrationActions(bool $commit)
    {
        foreach ($this->postMigrationsActions as $postMigrationsAction) {
            $postMigrationsAction->execute($commit);
        }
    }
}
