<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Definition;

use Dedi\SyliusMigrationPlugin\Dto\AfterMigrateDefinition;
use Dedi\SyliusMigrationPlugin\Dto\FilterMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\IdentifierMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\OrderMappingDefinition;
use Dedi\SyliusMigrationPlugin\Dto\SearchMappingDefinition;

class TableMappingDefinition
{
    private string $targetTableName;
    private string $sourceTableName;
    private array $identifierMappingDefinitions;
    private array $columnMappingDefinitions;
    private string $sql;
    private string $sourcePrimaryKey;
    private array $searchMapping;
    private array $filterMapping;
    private array $logginColumns;
    private ?OrderMappingDefinition $orderMappingDefinition;
    private ?AfterMigrateDefinition $afterMigrateDefinition;

    public function __construct(
        string $targetTableName,
        string $sourceTableName,
        string $sql,
        string $sourcePrimaryKey,
        array $identifierMappingDefinitions,
        array $columnMappingDefinitions,
        array $searchMapping,
        array $filterMapping,
        array $logginColumns,
        ?OrderMappingDefinition $orderMappingDefinition = null,
        ?AfterMigrateDefinition $afterMigrateDefinition = null
    ) {
        $this->targetTableName = $targetTableName;
        $this->sourceTableName = $sourceTableName;
        $this->sourcePrimaryKey = $sourcePrimaryKey;
        $this->columnMappingDefinitions = $columnMappingDefinitions;
        $this->sql = $sql;
        $this->searchMapping = $searchMapping;
        $this->filterMapping = $filterMapping;
        $this->orderMappingDefinition = $orderMappingDefinition;
        $this->logginColumns = $logginColumns;
        $this->identifierMappingDefinitions = $identifierMappingDefinitions;
        $this->afterMigrateDefinition = $afterMigrateDefinition;
    }

    public function getTargetTableName(): string
    {
        return $this->targetTableName;
    }

    public function getSourceTableName(): string
    {
        return $this->sourceTableName;
    }

    /**
     * @return array|ColumnMappingDefinition[]
     */
    public function getColumnMappingDefinitions(): array
    {
        return $this->columnMappingDefinitions;
    }

    public function getSql(): string
    {
        return $this->sql;
    }

    public function getSourcePrimaryKey(): string
    {
        return $this->sourcePrimaryKey;
    }

    /**
     * @return array|IdentifierMappingDefinition[]
     */
    public function getIdentifierMappingDefinitions(): array
    {
        return $this->identifierMappingDefinitions;
    }

    public function getAfterMigrateDefinition(): ?AfterMigrateDefinition
    {
        return $this->afterMigrateDefinition;
    }

    public function hasColumnMapping(string $columnName): bool
    {
        /** @var ColumnMappingDefinition $columnMappingDefinition */
        foreach ($this->columnMappingDefinitions as $columnMappingDefinition) {
            if ($columnName === $columnMappingDefinition->getTargetColumName()) {
                return true;
            }
        }

        return false;
    }

    /** @return array|SearchMappingDefinition[] */
    public function getSearchMapping(): array
    {
        return $this->searchMapping;
    }

    public function hasSearchMapping(): bool
    {
        return 0 < count($this->searchMapping);
    }

    /** @return array|FilterMappingDefinition[] */
    public function getFilterMapping(): array
    {
        return $this->filterMapping;
    }

    public function hasFilterMapping(): bool
    {
        return 0 < count($this->filterMapping);
    }

    public function getOrderMappingDefinition(): ?OrderMappingDefinition
    {
        return $this->orderMappingDefinition;
    }

    public function getLoggingColumns(): array
    {
        return $this->logginColumns;
    }
}
