<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Service\MigrationAction;

use Dedi\SyliusMigrationPlugin\Config\MigrationConfig;
use Dedi\SyliusMigrationPlugin\Dto\MigrationOption;
use Dedi\SyliusMigrationPlugin\Service\Logger\MigrationLoggerInterface;
use Doctrine\DBAL\Connection;
use Psr\Log\LogLevel;

/**
 * Exemple de config :
 *
 *   truncate_table_before_migration:
 *       - [nom_de_la_table]
 *       - ...
 */
class TruncateTableBeforeMigration extends AbstractMigrationAction implements PreMigrationActionInterface
{
    private Connection $connection;
    private MigrationLoggerInterface $logger;
    private MigrationConfig $migrationConfig;

    public function __construct(Connection $connection, MigrationLoggerInterface $logger, MigrationConfig $migrationConfig)
    {
        $this->connection = $connection;
        $this->logger = $logger;
        $this->migrationConfig = $migrationConfig;
    }

    public function getType(): string
    {
        return 'truncate_table_before_migration';
    }

    public function execute(MigrationOption $migrationOption): void
    {
        if (!$migrationOption->isTruncate()) {
            return;
        }

        if (!$migrationOption->isDryRun()) {
            $this->connection->executeQuery('SET FOREIGN_KEY_CHECKS = 0;');
        }

        $sql = sprintf('TRUNCATE TABLE `%s`', $this->migrationConfig->getTempMappingTableName());
        $this->logger->log(LogLevel::INFO, $sql);
        if (!$migrationOption->isDryRun()) {
            $this->connection->executeQuery($sql);
        }

        foreach ($this->options as $table) {
            $sql = sprintf('TRUNCATE TABLE `%s`', $table);
            $this->logger->log(LogLevel::INFO, $sql);

            if (!$migrationOption->isDryRun()) {
                $this->connection->executeQuery($sql);
            }
        }

        if (!$migrationOption->isDryRun()) {
            $this->connection->executeQuery('SET FOREIGN_KEY_CHECKS = 1;');
        }
    }
}
