<?php

declare(strict_types=1);

namespace Dedi\SyliusMigrationPlugin\Service\Sql;

use Dedi\SyliusMigrationPlugin\Config\MigrationConfig;
use Dedi\SyliusMigrationPlugin\Definition\TableMappingDefinition;
use Dedi\SyliusMigrationPlugin\Registry\TransformerRegistry;
use Dedi\SyliusMigrationPlugin\Service\Logger\MigrationLoggerInterface;
use Doctrine\DBAL\Connection;

abstract class AbstractSqlProcessor implements SqlProcessorInterface
{
    protected MigrationConfig $config;
    protected Connection $connection;
    protected TransformerRegistry $transformerRegistry;
    protected MigrationLoggerInterface $logger;

    public function __construct(
        MigrationConfig $config,
        Connection $connection,
        TransformerRegistry $transformerRegistry,
        MigrationLoggerInterface $logger
    ) {
        $this->config = $config;
        $this->connection = $connection;
        $this->transformerRegistry = $transformerRegistry;
        $this->logger = $logger;
    }

    public function tableExists(TableMappingDefinition $tableMappingDefinition, string $columnName): bool
    {
        if ('' === $columnName) {
            return true;
        }

        try {
            $this
                ->connection
                ->executeQuery(sprintf('DESCRIBE `%s`', $columnName))->fetchAllAssociative();

            return true;
        } catch (\Exception $e) {
            $this->logger->error($tableMappingDefinition, sprintf('Table %s does not exists in database %s', $columnName, $this->connection->getDatabase()));

            return false;
        }
    }

    public function defuse()
    {
        $this->config->setCommit(false);
        $this->connection->setAutoCommit(false);
    }
}
